"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const anymatch_1 = tslib_1.__importDefault(require("anymatch"));
const fs_1 = tslib_1.__importDefault(require("fs"));
const settings_1 = tslib_1.__importDefault(require("../../settings"));
const parser_1 = tslib_1.__importDefault(require("../../parser"));
exports.nodeIsImportDeclaration = (node) => node.type === "ImportDeclaration";
exports.nodeIsImportSpecifier = (node) => node.type === "ImportSpecifier";
exports.nodeIsImportDefaultSpecifier = (node) => node.type === "ImportDefaultSpecifier";
exports.nodeIsImportNamespaceSpecifier = (node) => node.type === "ImportNamespaceSpecifier";
exports.nodeIsMemberExpression = (node) => node.type === "MemberExpression";
class Cache {
    constructor(context) {
        this.settings = new settings_1.default(context);
        this.parser = new parser_1.default(this.settings);
    }
    static clear() {
        Cache.baseFilenameToClasses = {};
        Cache.specifierToClasses = {};
    }
    processImportDeclaration(node) {
        if (!exports.nodeIsImportDeclaration(node) ||
            typeof node.source.value !== "string") {
            return null;
        }
        const baseFilename = node.source.value;
        const filename = this.settings.resolveFile(baseFilename);
        if (!anymatch_1.default(this.settings.include, filename) ||
            anymatch_1.default(this.settings.exclude, filename) ||
            !fs_1.default.existsSync(filename)) {
            return null;
        }
        const explicitImports = [];
        let specifier = null;
        node.specifiers.forEach(spec => {
            if (exports.nodeIsImportSpecifier(spec)) {
                explicitImports.push(spec);
            }
            else if (exports.nodeIsImportDefaultSpecifier(spec) ||
                exports.nodeIsImportNamespaceSpecifier(node)) {
                specifier = spec.local.name;
            }
        });
        const classes = Cache.baseFilenameToClasses[baseFilename] !== undefined
            ? Cache.baseFilenameToClasses[baseFilename].classes
            : this.parser.parse(filename);
        if (specifier) {
            Cache.specifierToClasses[specifier] = {
                baseFilename,
                classes,
            };
        }
        if (Cache.baseFilenameToClasses[baseFilename] === undefined) {
            Cache.baseFilenameToClasses[baseFilename] = {
                node,
                classes,
            };
        }
        return {
            node,
            baseFilename,
            specifier,
            explicitImports,
            classes,
        };
    }
    processMemberExpression(node) {
        if (!exports.nodeIsMemberExpression(node)) {
            return null;
        }
        const objectName = node.object.name;
        if (Cache.specifierToClasses[objectName] === undefined) {
            return null;
        }
        const { baseFilename, classes } = Cache.specifierToClasses[objectName];
        const className = node.computed
            ? node.property.value
            : node.property.name;
        if (!className) {
            return null;
        }
        return {
            node,
            baseFilename,
            className: className.toString(),
            classes,
        };
    }
}
Cache.baseFilenameToClasses = {};
Cache.specifierToClasses = {};
exports.Cache = Cache;
